Package: BiocFileCache
Title: Manage Files Across Sessions
Version: 3.3.0
Authors@R: c(person("Lori", "Shepherd",
      email = "lori.shepherd@roswellpark.org",
      role = c("aut", "cre")),
    person("Martin", "Morgan",
      email = "martin.morgan@roswellpark.org",
      role = "aut"))
Description: This package creates a persistent on-disk cache of files
    that the user can add, update, and retrieve. It is useful for
    managing resources (such as custom Txdb objects) that are costly
    or difficult to create, web resources, and data files used across
    sessions.
Depends: R (>= 3.4.0), dbplyr (>= 1.0.0)
Imports: methods, stats, utils, dplyr, RSQLite, DBI, filelock, curl,
        httr2
BugReports: https://github.com/Bioconductor/BiocFileCache/issues
DevelopmentURL: https://github.com/Bioconductor/BiocFileCache
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.2
biocViews: DataImport
VignetteBuilder: knitr
Suggests: testthat, knitr, BiocStyle, rmarkdown, rtracklayer
git_url: https://git.bioconductor.org/packages/BiocFileCache
git_branch: devel
git_last_commit: 20482da
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:30:52 UTC; biocbuild
Author: Lori Shepherd [aut, cre],
  Martin Morgan [aut]
Maintainer: Lori Shepherd <lori.shepherd@roswellpark.org>
