% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeCachedActiveBinding.R
\name{makeCachedActiveBinding}
\alias{makeCachedActiveBinding}
\title{makeCachedActiveBinding}
\usage{
makeCachedActiveBinding(sym, fun, env = .GlobalEnv, verbose = FALSE)
}
\arguments{
\item{sym}{See \code{\link{makeActiveBinding}} in the \pkg{base}
package.}

\item{fun}{See \code{\link{makeActiveBinding}} in the \pkg{base}
package.}

\item{env}{See \code{\link{makeActiveBinding}} in the \pkg{base}
package.}

\item{verbose}{Set to TRUE to see caching in action (useful for
troubleshooting).}
}
\description{
Like \code{\link{makeActiveBinding}} but the value of the active
binding gets only evaluated once and is "remembered".
}
\examples{
makeCachedActiveBinding("x", function() runif(1), verbose=TRUE)
x
x
}
