% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BFCOption.R
\name{BFCOption}
\alias{BFCOption}
\alias{setBFCOption}
\alias{getBFCOption}
\title{BFCOption
These functions help get and set an R variable CACHE that controls the
default caching location.}
\usage{
setBFCOption(arg, value)
}
\arguments{
\item{arg}{character(1) option to get or set}

\item{value}{The value to be assigned to the designated option}
}
\value{
Value of request option or invisible successfully set option
}
\description{
BFCOption
These functions help get and set an R variable CACHE that controls the
default caching location.
}
\details{
Currently the only supported option is CACHE. This controls the default
location of the BiocFileCache caching directory. By default the value is
established by \code{tools::R_user_dir("BiocFileCache",which="cache")}. This
value can also be defaultly set by using system and global environment
variables visible \emph{before} the package is loaded. The variable that
should be set if utilized is \dQuote{BFC_CACHE}
}
\examples{
origPath = getBFCOption('CACHE')
\donttest{setBFCOption('CACHE', "~/.myBFC") }
}
\author{
Lori Shepherd
}
